import re
from datetime import date, timedelta

from django.contrib.messages import get_messages
from django.urls import reverse, reverse_lazy

import pytest
from bs4 import BeautifulSoup as bs

from benevalibre.association.models import Association, User
from benevalibre.association.views import (
    AssociationIndex,
    get_management_buttons,
)
from benevalibre.utils.tests import (
    RE_DELETE_BTN,
    ManagerOnlyViewMixin,
    count_text_in_content,
    is_login_view,
)

RE_URL = re.compile(r'http(|s)://[^/]*/([^ ]*/)$', re.MULTILINE)


@pytest.mark.django_db
class TestAssociationManagementButtons:
    def test_foreign(self, association, foreign):
        buttons = get_management_buttons(association, foreign)
        assert len(buttons) == 0

    def test_manager(self, association, user, manager):
        buttons = get_management_buttons(association, user)
        assert len(buttons) == 7

    def test_superuser(self, association, superuser):
        buttons = get_management_buttons(association, superuser)
        assert len(buttons) == 1

    @pytest.mark.parametrize(
        'role_name, expected',
        [('Bénévole', 0), ('Animat⋅eur⋅rice', 5), ('Dirigeant', 7)],
    )
    def test_role(
        self, association, user, foreign, engagement, role_name, expected
    ):
        # upgrade foreign pour avoir au moins toujours un dirigeant
        role = association.role_set.get_dirigeant()
        association.engagement_set.create(user=foreign, role=role)

        # bidouille l'engagement
        engagement = association.get_engagement(user)
        engagement.role = association.role_set.get(name=role_name)
        engagement.save()

        assert len(get_management_buttons(association, user)) == expected

    def test_role_list_users(self, association, user, foreign, engagement):
        # upgrade foreign pour avoir au moins toujours un dirigeant
        role = association.role_set.get_dirigeant()
        association.engagement_set.create(user=foreign, role=role)

        role = association.role_set.get_benevole()
        role.list_users = True
        role.save()

        engagement = association.get_engagement(user)
        engagement.role = role
        engagement.save()

        assert len(get_management_buttons(association, user)) == 1


@pytest.mark.django_db
class TestAssociationIndex:
    url = reverse_lazy('association:index')

    @pytest.fixture(autouse=True)
    def setup_associations(self):
        self.associations = [
            Association.objects.create(name="Asso 1"),
            Association.objects.create(name="Asso 2", conceal=True),
            Association.objects.create(name="Asso 3", is_active=False),
        ]

    def test_registration_closed(self, client, closed_registration):
        response = client.get(self.url)
        assert response.status_code == 200
        assert not count_text_in_content(response, "Ajouter une association")

    def test_anonymous(self, client):
        response = client.get(self.url)
        assert response.status_code == 200
        assert len(response.context['object_list']) == 1
        assert count_text_in_content(response, "Asso 1")
        assert count_text_in_content(response, "Ajouter une association")

    def test_user(self, client, user):
        client.force_login(user)
        response = client.get(self.url)
        assert response.status_code == 200
        assert len(response.context['object_list']) == 1
        assert count_text_in_content(response, "Asso 1")
        assert count_text_in_content(response, "Ajouter une association")

    def test_user_conceal(self, client, user):
        # on devient bénévole dans l'association cachée
        association = self.associations[1]
        association.engagement_set.create(
            user=user, role=association.role_set.last()
        )

        client.force_login(user)
        response = client.get(self.url)
        assert response.status_code == 200
        assert len(response.context['object_list']) == 2
        assert count_text_in_content(response, "Asso 2")

    def test_user_inactive(self, client, user):
        # on devient bénévole dans l'association en modération
        association = self.associations[2]
        association.engagement_set.create(
            user=user, role=association.role_set.last()
        )

        client.force_login(user)
        response = client.get(self.url)
        assert response.status_code == 200
        assert len(response.context['object_list']) == 2
        assert count_text_in_content(response, "Asso 3")
        assert count_text_in_content(response, "En modération")

    def test_superuser(self, client, superuser):
        client.force_login(superuser)

        response = client.get(self.url)
        assert response.status_code == 200
        assert len(response.context['object_list']) == 3
        assert count_text_in_content(response, "En modération")

    def test_search(self, client):
        Association.objects.create(name="Asso 4"),

        response = client.get(self.url, {'search': 'Asso'})
        assert response.status_code == 200
        assert len(response.context['object_list']) == 2
        assert count_text_in_content(response, "Asso 1")
        assert count_text_in_content(response, "Asso 4")

        response = client.get(self.url, {'search': 'Asso 1'})
        assert response.status_code == 200
        assert len(response.context['object_list']) == 1
        assert count_text_in_content(response, "Asso 1")
        assert not count_text_in_content(response, "Asso 4")

        response = client.get(self.url, {'search': 'Asso 6'})
        assert response.status_code == 200
        assert len(response.context['object_list']) == 0

    def test_asso_count(self):
        asso_index = AssociationIndex()
        # Les assos de tests sont créées plus haut (setup_associations)
        # Il y a donc 2 assos actives et 1 inactive
        assert asso_index.get_asso_count() == 2

    def test_user_count(self):
        asso_index = AssociationIndex()
        for i in range(0, 20):
            user = User.objects.create_user(
                "user{}@test.com".format(i), password='test'
            )
            user.save()
        assert asso_index.get_user_count() == 20

    def test_benevalo_count(self, user, association, engagement, category):
        asso_index = AssociationIndex()
        user.benevalo_set.create(
            title="test 1",
            duration=timedelta(0, 1520),
            distance=21,
            execution_date=date(2020, 8, 2),
            association=association,
            category=category,
        )
        user.benevalo_set.create(
            title="test 2",
            duration=timedelta(0, 3950),
            distance=15,
            execution_date=date(2019, 2, 25),
            association=association,
            category=category,
        )
        assert asso_index.get_benevalo_count() == 2


@pytest.mark.django_db
class TestAssociationDetail:
    @classmethod
    @pytest.fixture(autouse=True)
    def setup_class(cls, association):
        cls.association = association
        cls.url = reverse('association:detail', args=[cls.association.id])

    def test_get(self, client, user):
        response = client.get(self.url)
        assert response.status_code == 200
        assert count_text_in_content(response, "association loi de 1901")
        assert count_text_in_content(response, "Connectez-vous")
        assert not count_text_in_content(response, "Rejoindre")

        client.force_login(user)
        response = client.get(self.url)
        assert response.status_code == 200
        assert count_text_in_content(response, "Rejoindre")
        assert response.context['benevalo_table'] is None

    def test_get_with_engagement(self, client, user, engagement):
        client.force_login(user)
        response = client.get(self.url)
        assert response.status_code == 200
        assert not count_text_in_content(response, "Rejoindre")
        assert count_text_in_content(response, "Mes actions de bénévolat")

    def test_get_conceal(self, client, user, superuser):
        self.association.conceal = True
        self.association.save()

        response = client.get(self.url)
        assert response.status_code == 404

        client.force_login(user)
        response = client.get(self.url)
        assert response.status_code == 404

        client.force_login(superuser)
        response = client.get(self.url)
        assert response.status_code == 200

    def test_get_conceal_with_engagement(self, client, user, engagement):
        self.association.conceal = True
        self.association.save()

        client.force_login(user)
        response = client.get(self.url)
        assert response.status_code == 200

    def test_titlebuttons_foreign(self, client, association, foreign):
        client.force_login(foreign)
        response = client.get(
            reverse('association:detail', args=[association.id])
        )
        assert not count_text_in_content(response, "title-buttons")
        assert not count_text_in_content(response, "Gérer")

    def test_titlebuttons_manager(self, client, association, user, manager):
        client.force_login(user)
        response = client.get(
            reverse('association:detail', args=[association.id])
        )
        assert count_text_in_content(response, "title-buttons")
        assert count_text_in_content(response, "Gérer")

        buttons = (
            bs(response.content, 'html.parser')
            .find(class_='title-buttons')
            .find_all('a', class_='dropdown-item')
        )
        assert len(buttons) == 7


@pytest.mark.django_db
class TestAssociationUpdate:
    @pytest.mark.parametrize(
        'use, expected',
        [(None, 200), ('user', 200), ('staff', 200), ('foreign', 404)],
    )
    def test_management_access(
        self,
        client,
        association,
        user,
        superuser,
        foreign,
        manager,
        use,
        expected,
    ):
        if use == 'staff':
            client.force_login(superuser)
        elif use == 'foreign':
            client.force_login(foreign)
        elif use:
            client.force_login(user)

        response = client.get(
            reverse('association:update', args=[association.id]), follow=True
        )
        assert response.status_code == expected
        if not use:
            assert is_login_view(response)


@pytest.mark.django_db
class TestAssociationDelete:
    @pytest.mark.parametrize(
        'use, expected',
        [(None, 200), ('user', 200), ('staff', 200), ('foreign', 404)],
    )
    def test_deletion_access(
        self,
        client,
        association,
        user,
        superuser,
        foreign,
        manager,
        use,
        expected,
    ):
        if use == 'staff':
            client.force_login(superuser)
        elif use == 'foreign':
            client.force_login(foreign)
        elif use:
            client.force_login(user)

        response = client.get(
            reverse('association:delete', args=[association.id]), follow=True
        )
        assert response.status_code == expected
        if not use:
            assert is_login_view(response)


@pytest.mark.django_db
class TestAssociationCreate:
    create_url = reverse_lazy('association:create')

    def test_create_registration_closed(self, client, closed_registration):
        response = client.get(self.create_url)
        assert response.status_code == 404

    def test_create_update_delete(self, client, user):
        client.force_login(user)

        # 1. on créé
        with open('benevalibre/utils/tests/images/flower.jpg', 'rb') as logo:
            data = {'name': "L'AssoTest", 'logo': logo}
            response = client.post(self.create_url, data, follow=True)

        assert response.status_code == 200
        assert count_text_in_content(response, "AssoTest")
        assert count_text_in_content(response, "Secteur d'activité")

        messages = [m.message for m in get_messages(response.wsgi_request)]
        assert len(messages) == 1
        assert "a été ajouté" in messages[0]

        assert (
            user.engagement_set.active()
            .filter(role__manage_association=True)
            .exists()
        )

        buttons = (
            bs(response.content, 'html.parser')
            .find(class_='title-buttons')
            .find_all('a', class_='dropdown-item')
        )
        assert len(buttons) == 7

        # 2. on mets à jour
        url = [b.attrs['href'] for b in buttons if "Modifier" in str(b)][0]
        data = {'name': "L'AssoTest", 'description': 'Une asso qui en jette'}
        response = client.post(url, data, follow=True)

        assert response.status_code == 200
        assert count_text_in_content(response, "qui en jette")

        messages = [m.message for m in get_messages(response.wsgi_request)]
        assert len(messages) == 1
        assert "a été modifié" in messages[0]

        # 3. on supprime
        response = client.get(url, follow=True)
        url = (
            bs(response.content, 'html.parser')
            .find('a', string=RE_DELETE_BTN)
            .attrs['href']
        )
        data = {'confirm': True}
        response = client.post(url, data, follow=True)
        assert response.status_code == 200

        messages = [m.message for m in get_messages(response.wsgi_request)]
        assert len(messages) == 1
        assert "a été supprimé" in messages[0]

    def test_create_twice(self, client, association, user):
        client.force_login(user)

        # 1. on tente de créer
        data = {"name": "L'AssoTest"}
        response = client.post(self.create_url, data, follow=True)

        assert response.status_code == 200
        assert count_text_in_content(response, "Ajouter une association")

        messages = [m.message for m in get_messages(response.wsgi_request)]
        assert len(messages) == 1
        assert "Une association existe déjà sous ce nom." in messages[0]

        # 2. on force la création
        data = {"name": "L'AssoTest"}
        response = client.post(self.create_url, data, follow=True)

        assert response.status_code == 200

        messages = [m.message for m in get_messages(response.wsgi_request)]
        assert len(messages) == 1
        assert "a été ajouté" in messages[0]

    def test_moderated_create(
        self,
        mailoutbox,
        client,
        user,
        foreign,
        superuser,
        moderated_registration,
    ):
        board_url = reverse_lazy('base:board')

        assert not user.is_superuser
        client.force_login(user)

        # 1. on demande la création
        data = {'name': "L'AssoTest"}
        response = client.post(self.create_url, data, follow=True)
        assert response.status_code == 200
        assert count_text_in_content(response, "Mes engagements associatifs")
        assert not count_text_in_content(response, "Secteur d'activité")
        assert "L'AssoTest" not in bs(response.content, 'html.parser').find(
            'tbody'
        ).find_all('tr')

        messages = [m.message for m in get_messages(response.wsgi_request)]
        assert len(messages) == 1
        assert "modération" in messages[0]

        # 2. un super user a reçu un email
        assert len(mailoutbox) == 1
        assert superuser.email in mailoutbox[0].to
        assert superuser.get_short_name() in mailoutbox[0].body
        assert "L'AssoTest" in mailoutbox[0].body
        assert "Activation d'un nouveau compte asso" in mailoutbox[0].subject
        moderation_url = RE_URL.search(mailoutbox[0].body).group()
        assert moderation_url

        # 3. on vérifie qu'on a accès
        association = user.engagement_set.active().last().association
        assert association.name == "L'AssoTest"
        assert not association.is_active

        response = client.get(
            reverse('association:detail', args=[association.id])
        )
        assert response.status_code == 200

        # 4. on vérifie qu'un autre n'y a pas accès
        client.logout()
        client.force_login(foreign)
        response = client.get(
            reverse('association:detail', args=[association.id])
        )
        assert response.status_code == 404

        # 5. elle est visible à un superuser dans son board
        client.logout()
        client.force_login(superuser)
        response = client.get(board_url)
        assert count_text_in_content(response, "Modérations d'associations")

        table_row = (
            bs(response.content, 'html.parser').find('tbody').find_all('tr')
        )
        assert len(table_row) == 1
        assert "L'AssoTest" in str(table_row[0])
        assert "user@example.org" in str(table_row[0])
        assert "(en modération)" in str(table_row[0])

        # 6. elle est aussi visible à un superuser en suivant le lien fourni
        # par email
        via_email_response = client.get(moderation_url)
        assert count_text_in_content(
            via_email_response, "Modifier une association"
        )

        # 7. un superuser donne suite à la demande depuis le board
        url = (
            bs(response.content, 'html.parser')
            .find('tbody')
            .find_all('a', class_='update-link')[-1]
            .attrs['href']
        )
        response = client.get(url)
        assert count_text_in_content(response, "Modifier une association")

        # on prend tout et on post
        form = bs(response.content, 'html.parser').find("form")
        input_kv = [
            (i.get('name'), i.get('value'))
            for i in form.find_all('input')
            if i.get('value')
        ]
        select_kv = [
            (s.parent.get('name'), s.get('value'))
            for s in form.find_all('option', attrs={"selected": ""})
            if s.get('value')
        ]
        data = dict(input_kv + select_kv)

        response = client.post(url, data, follow=True)
        assert not count_text_in_content(
            response, "Modérations d'associations"
        )

        # 8. le bénévole a désormais accès
        client.logout()
        client.force_login(user)
        response = client.get(board_url)
        assert count_text_in_content(response, "Mes engagements associatifs")

        table_row = (
            bs(response.content, 'html.parser').find('tbody').find_all('tr')
        )
        assert len(table_row) == 1
        assert "L'AssoTest" in str(table_row[0])
        assert "Dirigeant" in str(table_row[0])
        assert "(en  modération)" not in str(table_row[0])
        assert "disabled" not in str(table_row[0])


@pytest.mark.django_db
class TestProjet(ManagerOnlyViewMixin):
    title = "Liste des projets"

    @classmethod
    @pytest.fixture(autouse=True)
    def setup_class(cls, association):
        cls.association = association
        cls.url = reverse(
            'association:project:index', args=[cls.association.id]
        )

    def test_post_create_update_delete(self, client, user, manager):
        client.force_login(user)

        # 1. on liste les projets de l'asso
        response = client.get(self.url)
        assert response.status_code == 200
        table_row = (
            bs(response.content, 'html.parser').find('tbody').find_all('tr')
        )
        assert len(table_row) == 1
        assert "Aucune donnée disponible." in str(table_row[-1])

        # 2. on créé un projet dans l'asso
        url = (
            bs(response.content, 'html.parser')
            .find(class_='title-buttons')
            .find('a')
            .attrs['href']
        )
        data = {'name': 'project de test'}
        response = client.post(url, data, follow=True)
        assert response.status_code == 200
        assert count_text_in_content(response, "project de test")
        messages = [m.message for m in get_messages(response.wsgi_request)]
        assert len(messages) == 1
        assert "« project de test » a été ajouté" in messages[0]

        table_row = (
            bs(response.content, 'html.parser').find('tbody').find_all('tr')
        )
        assert len(table_row) == 1
        assert "project de test" in str(table_row[-1])

        # 3. on modifie
        url = (
            bs(response.content, 'html.parser')
            .find('tbody')
            .find_all('a', class_='update-link')[-1]
            .attrs['href']
        )
        data = {'name': 'project de lutte'}
        response = client.post(url, data, follow=True)
        assert response.status_code == 200

        assert not count_text_in_content(response, "project de test")
        assert count_text_in_content(response, "project de lutte")
        messages = [m.message for m in get_messages(response.wsgi_request)]
        assert len(messages) == 1
        assert "a été modifié" in messages[0]

        table_row = (
            bs(response.content, 'html.parser').find('tbody').find_all('tr')
        )
        assert len(table_row) == 1
        assert "project de lutte" in str(table_row[-1])

        # 4. on supprime
        response = client.get(url, follow=True)
        url = (
            bs(response.content, 'html.parser')
            .find('a', string=RE_DELETE_BTN)
            .attrs['href']
        )
        data = {'confirm': True}
        response = client.post(url, data, follow=True)
        assert response.status_code == 200

        messages = [m.message for m in get_messages(response.wsgi_request)]
        assert len(messages) == 1
        assert "a été supprimé" in messages[0]

        table_row = (
            bs(response.content, 'html.parser').find('tbody').find_all('tr')
        )
        assert len(table_row) == 1
        assert "Aucune donnée disponible." in str(table_row[-1])


@pytest.mark.django_db
class TestEngagement(ManagerOnlyViewMixin):
    title = "Liste des bénévoles"

    @classmethod
    @pytest.fixture(autouse=True)
    def setup_class(cls, association):
        cls.association = association
        cls.url = reverse(
            'association:engagement:index', args=[cls.association.id]
        )

    def test_post_create_update_delete(
        self, client, mailoutbox, user, manager, foreign
    ):
        client.force_login(user)

        dirigeant_role = self.association.role_set.get_dirigeant()
        benevole_role = self.association.role_set.get(name="Bénévole")
        animateur_role = self.association.role_set.get(name="Animat⋅eur⋅rice")

        # 1. on liste les engagements de l'asso
        response = client.get(self.url)
        assert response.status_code == 200

        table_row = (
            bs(response.content, 'html.parser').find('tbody').find_all('tr')
        )
        assert len(table_row) == 1
        assert "Dirigeant" in str(table_row[-1])

        # 2. on échoue à modifier notre rôle
        url = (
            bs(response.content, 'html.parser')
            .find('tbody')
            .find_all('a', class_='update-link')[-1]
            .attrs['href']
        )
        data = {'role': animateur_role.id}
        response = client.post(url, data, follow=True)
        assert response.status_code == 200
        messages = [m.message for m in get_messages(response.wsgi_request)]
        assert len(messages) == 0
        assert count_text_in_content(response, "rendrait impossible la gesti")

        # 3. on modifie notre rôle

        # 3.1 on créé un nouveau dirigeant
        engagement = self.association.engagement_set.create(
            user=foreign, role=dirigeant_role
        )

        # 3.2 on arrive à modifier notre rôle
        data = {'role': animateur_role.id}
        response = client.post(url, data, follow=True)
        assert response.status_code == 200
        messages = [m.message for m in get_messages(response.wsgi_request)]
        assert len(messages) == 1
        assert "a été modifié" in messages[0]

        table_row = (
            bs(response.content, 'html.parser').find('tbody').find_all('tr')
        )
        assert len(table_row) == 2
        assert "Animat⋅eur⋅rice" in str(table_row[-2])

        # 4. on tente de redevenir dirigeant mais on n'est plus en capacité
        url = (
            bs(response.content, 'html.parser')
            .find('tbody')
            .find_all('a', class_='update-link')[-2]
            .attrs['href']
        )

        data = {'role': dirigeant_role.id}
        response = client.post(url, data, follow=True)
        assert response.status_code == 200

        form = bs(response.content, 'html.parser').find('form')
        assert "Ce choix ne fait pas partie de ceux disponibles." in str(form)

        options = form.find_all('option')
        assert len(options) == 3

        assert [o.attrs['value'] for o in options] == [
            '',
            str(benevole_role.id),
            str(animateur_role.id),
        ]

        # 5. on tente de modifier un dirigeant mais on n'est pas en capacité
        url2 = reverse(
            'association:engagement:update',
            args=[self.association.id, engagement.id],
        )
        response = client.get(url2, follow=True)
        options = (
            bs(response.content, 'html.parser').find('form').find_all('option')
        )

        assert len(options) == 2
        assert [o.attrs['value'] for o in options] == [
            '',
            str(dirigeant_role.id),
        ]

        # 6. un dirigeant nous modifie et on reçoit une notification
        client.logout()
        client.force_login(foreign)
        data = {'role': dirigeant_role.id}
        response = client.post(url, data, follow=True)
        assert len(mailoutbox) == 1
        assert user.email in mailoutbox[0].to
        assert user.get_short_name() in mailoutbox[0].body
        assert self.association.name in mailoutbox[0].body
        assert "Mise à jour de votre engagement" in mailoutbox[0].subject

    def test_self_erase_do_not_create_orpheans(self, client, user, manager):
        client.force_login(user)

        assert self.association.can_manage_engagements(user)

        data = {'confirm': True}
        response = client.post(
            reverse(
                'association:engagement:delete',
                args=[self.association.id, user.engagement_set.first().id],
            ),
            data,
            follow=True,
        )

        assert response.status_code == 200
        assert (
            response.resolver_match.view_name
            == 'association:engagement:delete'
        )

        assert count_text_in_content(response, "Impossible de supprimer")
        assert count_text_in_content(
            response, "rendrait impossible la gestion"
        )

    def test_self_erase_go_back_to_home(self, client, foreign, user, manager):
        # we need to define the next leader before leaving
        r = self.association.role_set.get_dirigeant()
        self.association.engagement_set.create(user=foreign, role=r)

        assert self.association.can_manage_engagements(user)
        assert self.association.can_manage_engagements(foreign)

        client.force_login(user)

        data = {'confirm': True}
        response = client.post(
            reverse(
                'association:engagement:delete',
                args=[self.association.id, user.engagement_set.first().id],
            ),
            data,
            follow=True,
        )

        assert not self.association.can_manage_engagements(user)

        assert response.status_code == 200
        assert response.resolver_match.view_name == 'base:board'

        messages = [m.message for m in get_messages(response.wsgi_request)]
        assert len(messages) == 1
        assert "a été supprimé" in messages[0]


@pytest.mark.django_db
class TestLevel(ManagerOnlyViewMixin):
    title = "Liste des niveaux"

    @classmethod
    @pytest.fixture(autouse=True)
    def setup_class(cls, association):
        cls.association = association
        cls.url = reverse('association:level:index', args=[cls.association.id])

    def test_post_create_update_delete(self, client, user, manager):
        client.force_login(user)

        # 1. on liste les niveaux de l'asso
        response = client.get(self.url)
        assert response.status_code == 200
        table_row = (
            bs(response.content, 'html.parser').find('tbody').find_all('tr')
        )
        assert len(table_row) == 1
        assert "Aucune donnée disponible." in str(table_row[-1])

        # 2. on créé un niveau dans l'asso
        url = (
            bs(response.content, 'html.parser')
            .find(class_='title-buttons')
            .find('a')
            .attrs['href']
        )
        data = {'name': 'level de test'}
        response = client.post(url, data, follow=True)
        assert response.status_code == 200
        assert count_text_in_content(response, "level de test")
        messages = [m.message for m in get_messages(response.wsgi_request)]
        assert len(messages) == 1
        assert "« level de test » a été ajouté" in messages[0]

        table_row = (
            bs(response.content, 'html.parser').find('tbody').find_all('tr')
        )
        assert len(table_row) == 1
        assert "level de test" in str(table_row[-1])

        # 3. on modifie
        url = (
            bs(response.content, 'html.parser')
            .find('tbody')
            .find_all('a', class_='update-link')[-1]
            .attrs['href']
        )
        data = {'name': 'level de lutte'}
        response = client.post(url, data, follow=True)
        assert response.status_code == 200

        assert not count_text_in_content(response, "level de test")
        assert count_text_in_content(response, "level de lutte")
        messages = [m.message for m in get_messages(response.wsgi_request)]
        assert len(messages) == 1
        assert "a été modifié" in messages[0]

        table_row = (
            bs(response.content, 'html.parser').find('tbody').find_all('tr')
        )
        assert len(table_row) == 1
        assert "level de lutte" in str(table_row[-1])

        # 4. on supprime
        response = client.get(url, follow=True)
        url = (
            bs(response.content, 'html.parser')
            .find('a', string=RE_DELETE_BTN)
            .attrs['href']
        )
        data = {'confirm': True}
        response = client.post(url, data, follow=True)
        assert response.status_code == 200

        messages = [m.message for m in get_messages(response.wsgi_request)]
        assert len(messages) == 1
        assert "a été supprimé" in messages[0]

        table_row = (
            bs(response.content, 'html.parser').find('tbody').find_all('tr')
        )
        assert len(table_row) == 1
        assert "Aucune donnée disponible." in str(table_row[-1])


@pytest.mark.django_db
class TestRole(ManagerOnlyViewMixin):
    title = "Liste des rôles"

    @classmethod
    @pytest.fixture(autouse=True)
    def setup_class(cls, association):
        cls.association = association
        cls.url = reverse('association:role:index', args=[cls.association.id])

    def test_post_create_update_delete(self, client, user, manager):
        client.force_login(user)

        # 1. on liste les roles de l'asso
        response = client.get(self.url)
        assert response.status_code == 200

        table_row = (
            bs(response.content, 'html.parser').find('tbody').find_all('tr')
        )
        assert len(table_row) == 3

        # 2. on créé un role dans l'asso
        url = (
            bs(response.content, 'html.parser')
            .find(class_='title-buttons')
            .find('a')
            .attrs['href']
        )
        data = {'name': 'role de test', 'instance_role': 1}
        response = client.post(url, data, follow=True)
        assert response.status_code == 200
        assert count_text_in_content(response, "role de test")
        messages = [m.message for m in get_messages(response.wsgi_request)]
        assert len(messages) == 1
        assert "« role de test » a été ajouté" in messages[0]

        table_row = (
            bs(response.content, 'html.parser').find('tbody').find_all('tr')
        )
        assert len(table_row) == 4
        assert "role de test" in str(table_row[-1])

        # 3. on modifie
        url = (
            bs(response.content, 'html.parser')
            .find('tbody')
            .find_all('a', class_='update-link')[-1]
            .attrs['href']
        )
        data = {'name': 'role de lutte', 'instance_role': 1}
        response = client.post(url, data, follow=True)
        assert response.status_code == 200
        assert not count_text_in_content(response, "role de test")
        assert count_text_in_content(response, "role de lutte")
        messages = [m.message for m in get_messages(response.wsgi_request)]
        assert len(messages) == 1
        assert "a été modifié" in messages[0]

        table_row = (
            bs(response.content, 'html.parser').find('tbody').find_all('tr')
        )
        assert len(table_row) == 4
        assert "role de lutte" in str(table_row[-1])

        # 4. on supprime
        response = client.get(url, follow=True)
        url = (
            bs(response.content, 'html.parser')
            .find('a', string=RE_DELETE_BTN)
            .attrs['href']
        )
        data = {'confirm': True}
        response = client.post(url, data, follow=True)
        assert response.status_code == 200

        messages = [m.message for m in get_messages(response.wsgi_request)]
        assert len(messages) == 1
        assert "a été supprimé" in messages[0]

        table_row = (
            bs(response.content, 'html.parser').find('tbody').find_all('tr')
        )
        assert len(table_row) == 3

    def test_self_downgrade_dont_create_orphans(self, client, user, manager):
        client.force_login(user)

        assert self.association.can_manage_engagements(user)

        data = {'name': 'underground'}
        response = client.post(
            reverse(
                'association:role:update',
                args=[self.association.id, manager.role.id],
            ),
            data,
            follow=True,
        )

        assert self.association.can_manage_engagements(user)

        assert response.status_code == 200
        assert response.resolver_match.view_name == 'association:role:update'

        assert count_text_in_content(
            response, "rendrait impossible la gestion"
        )

    def test_self_downgrade_go_back_to_home(
        self, client, user, foreign, manager
    ):
        # we need to define the next leader before leaving
        r = self.association.role_set.get_benevole()
        r.manage_association = True
        r.save()
        self.association.engagement_set.create(user=foreign, role=r)

        assert self.association.can_manage_engagements(user)
        assert self.association.can_manage_engagements(foreign)

        client.force_login(user)

        data = {'name': 'underground'}
        response = client.post(
            reverse(
                'association:role:update',
                args=[self.association.id, manager.role.id],
            ),
            data,
            follow=True,
        )

        assert not self.association.can_manage_engagements(user)

        assert response.status_code == 200
        assert response.resolver_match.view_name == 'base:board'

        messages = [m.message for m in get_messages(response.wsgi_request)]
        assert len(messages) == 1
        assert "a été modifié" in messages[0]

    def test_self_erase_protected(self, client, user, manager):
        client.force_login(user)

        assert self.association.can_manage_roles(user)

        data = {'confirm': True}
        response = client.post(
            reverse(
                'association:role:delete',
                args=[self.association.id, manager.role.id],
            ),
            data,
            follow=True,
        )

        assert self.association.can_manage_roles(user)
        assert response.status_code == 200
        assert count_text_in_content(response, "actuellement référencé")
        assert count_text_in_content(response, "intégrité des données")

    def test_duplicate_new_comer(self, client, user, manager):
        client.force_login(user)

        data = {'name': 'role de test', 'new_comers': 1}
        response = client.post(
            reverse('association:role:create', args=[self.association.id]),
            data,
        )
        assert response.status_code == 200
        assert response.context['form'].errors
        assert count_text_in_content(response, "attribut « rôle par défaut »")


@pytest.mark.django_db
class TestCategory(ManagerOnlyViewMixin):
    title = "Liste des catégories"

    @classmethod
    @pytest.fixture(autouse=True)
    def setup_class(cls, association):
        cls.association = association
        cls.url = reverse(
            'association:category:index', args=[cls.association.id]
        )

    def test_post_create_update_delete(self, client, user, manager):
        client.force_login(user)

        # 1. on liste les categories de l'asso
        response = client.get(self.url)
        assert response.status_code == 200

        table_row = (
            bs(response.content, 'html.parser').find('tbody').find_all('tr')
        )
        assert len(table_row) == 12

        # 2. on créé une categorie dans l'asso
        url = (
            bs(response.content, 'html.parser')
            .find(class_='title-buttons')
            .find('a')
            .attrs['href']
        )
        data = {'name': 'category de test', 'instance_category': 1}
        response = client.post(url, data, follow=True)
        assert response.status_code == 200
        assert count_text_in_content(response, "category de test")
        messages = [m.message for m in get_messages(response.wsgi_request)]
        assert len(messages) == 1
        assert "« category de test » a été ajouté" in messages[0]

        table_row = (
            bs(response.content, 'html.parser').find('tbody').find_all('tr')
        )
        assert len(table_row) == 13
        assert "category de test" in str(table_row[-1])

        # 3. on modifie
        url = (
            bs(response.content, 'html.parser')
            .find('tbody')
            .find_all('a', class_='update-link')[-1]
            .attrs['href']
        )
        data = {'name': 'category de lutte', 'instance_category': 1}
        response = client.post(url, data, follow=True)
        assert response.status_code == 200
        assert not count_text_in_content(response, "category de test")
        assert count_text_in_content(response, "category de lutte")
        messages = [m.message for m in get_messages(response.wsgi_request)]
        assert len(messages) == 1
        assert "a été modifié" in messages[0]

        table_row = (
            bs(response.content, 'html.parser').find('tbody').find_all('tr')
        )
        assert len(table_row) == 13
        assert "category de lutte" in str(table_row[-1])

        # 4. on supprime
        response = client.get(url, follow=True)
        url = (
            bs(response.content, 'html.parser')
            .find('a', string=RE_DELETE_BTN)
            .attrs['href']
        )
        data = {'confirm': True}
        response = client.post(url, data, follow=True)
        assert response.status_code == 200

        messages = [m.message for m in get_messages(response.wsgi_request)]
        assert len(messages) == 1
        assert "a été supprimé" in messages[0]

        table_row = (
            bs(response.content, 'html.parser').find('tbody').find_all('tr')
        )
        assert len(table_row) == 12
