from datetime import timedelta

from django.core.exceptions import ValidationError

import pytest
import reversion
from reversion.models import Version

from benevalibre.benevalo import models


@pytest.mark.django_db
class TestCreateBenevalo:
    def test_create_benevalo(
        self, user, engagement, association, level, category
    ):
        benevalo = models.Benevalo.objects.create(
            distance=42,
            user=user,
            level=level,
            association=association,
            category=category,
        )
        assert set(models.Benevalo.objects.all()) == set([benevalo])

        benevalo = models.Benevalo.objects.create(
            duration=timedelta(minutes=70),
            user=user,
            level=level,
            association=association,
            category=category,
        )
        assert models.Benevalo.objects.last() == benevalo

    def test_create_benevalo_version(
        self, user, association, engagement, level, category
    ):
        with reversion.create_revision():
            benevalo = models.Benevalo.objects.create(
                distance=42,
                user=user,
                level=level,
                association=association,
                category=category,
            )
        assert Version.objects.get_for_object(benevalo).count() == 1

    def test_raise_exception_on_data_error(
        self, user, association, level, category
    ):
        """
        Une action de bénévolat devrait a minima comporter une durée ou une
        distance.
        """
        with pytest.raises(ValidationError):
            models.Benevalo.objects.create(
                user=user,
                level=level,
                association=association,
                category=category,
            )

        with pytest.raises(ValidationError):
            models.Benevalo.objects.create(
                # la durée doit être positive
                duration=timedelta(minutes=-70),
                user=user,
                level=level,
                association=association,
                category=category,
            )

    def test_raise_exception_on_data_missing(
        self, user, association, level, category
    ):
        """
        La méthode clean pouvait échouer sur des données trop pauvres
        """
        benevalo = models.Benevalo(user=user, level=level, category=category)
        with pytest.raises(ValidationError):
            benevalo.clean()

        benevalo = models.Benevalo(
            user=user, association=association, category=category
        )
        with pytest.raises(ValidationError):
            benevalo.clean()

        benevalo = models.Benevalo(
            user=user, level=level, association=association
        )
        with pytest.raises(ValidationError):
            benevalo.clean()

        benevalo = models.Benevalo()
        with pytest.raises(ValidationError):
            benevalo.clean()

    @pytest.mark.parametrize(
        'related',
        (
            ('level', 'niveau'),
            ('category', 'catégorie'),
            ('project', 'projet'),
        ),
    )
    def test_raise_exception_on_ownership_error(
        self, user, association, level, category, project, related
    ):
        """
        Une action de bénévolat devra toujours avoir la même parenté à
        l'association que les projets, niveaux et catégories liées.
        """
        # spoof the related association
        dummy = association._meta.model.objects.create(name='blang')
        eval(related[0]).association = dummy

        with pytest.raises(ValidationError, match=related[1]):
            models.Benevalo.objects.create(
                distance=42,
                user=user,
                level=level,
                association=association,
                category=category,
                project=project,
            )

    def test_raise_exception_on_engagement_error(
        self, user, association, level, category, project
    ):
        """
        Une action de bénévolat doit être lié à un⋅e bénévole de l'association.
        """
        # drop the related engagement
        user.engagement_set.all().delete()

        with pytest.raises(ValidationError, match="bénévole"):
            models.Benevalo.objects.create(
                distance=42,
                user=user,
                level=level,
                association=association,
                category=category,
                project=project,
            )


@pytest.mark.django_db
class TestWiewableBenevalo:
    def test_unviewable_benevalo(
        self, user, foreign, association, level, category, project, engagement
    ):
        models.Benevalo.objects.create(
            distance=42,
            user=user,
            level=level,
            association=association,
            category=category,
            project=project,
        )

        assert not models.Benevalo.objects.viewable_by(foreign).exists()

        foreign.engagement_set.create(
            association=association,
            role=association.role_set.get(name='Animat⋅eur⋅rice'),
        )

        assert set(models.Benevalo.objects.viewable_by(foreign)) == set(
            models.Benevalo.objects.viewable_by(user)
        )
