from datetime import date

from django.contrib.messages import get_messages
from django.urls import reverse_lazy

import pytest
from bs4 import BeautifulSoup as bs
from reversion.models import Version

from benevalibre.benevalo.models import Benevalo
from benevalibre.utils.tests import (
    RE_DELETE_BTN,
    StaffOnlyViewMixin,
    count_text_in_content,
)


@pytest.mark.django_db
class TestBenevalo(StaffOnlyViewMixin):
    url = reverse_lazy('benevalo:index')
    title = "Liste des actions de bénévolats"

    def test_post_create_update_delete(
        self,
        client,
        user,
        superuser,
        association,
        engagement,
        project,
        level,
        category,
    ):
        # 0. on fetch
        client.force_login(superuser)
        response = client.get(self.url)

        # 1. on tente la création sans catégorie
        url = (
            bs(response.content, 'html.parser')
            .find(class_='title-buttons')
            .find('a')
            .attrs['href']
        )
        data = {
            'user': user.id,
            'title': 'test',
            'duration': '33',
            'execution_date': date.today(),
            'association': association.id,
            'level': level.id,
        }
        response = client.post(url, data, follow=True)
        assert response.status_code == 200
        assert count_text_in_content(response, "Veuillez corriger les erreurs")

        # 1.1 on créé
        data['category'] = category.id
        response = client.post(url, data, follow=True)
        assert response.status_code == 200
        assert count_text_in_content(response, "AssoTest")
        assert count_text_in_content(response, "NiveauTest")
        assert count_text_in_content(response, "CatégorieTest")

        messages = [m.message for m in get_messages(response.wsgi_request)]
        assert len(messages) == 1
        assert "a été ajouté" in messages[0]

        benevalos = Benevalo.objects.all()
        assert len(benevalos) == 1
        assert Version.objects.get_for_object(benevalos[0]).count() == 1

        # 2. on update
        url = (
            bs(response.content, 'html.parser')
            .find('tbody')
            .find_all('a', class_='update-link')[-1]
            .attrs['href']
        )
        data = {
            'user': user.id,
            'duration': '33',
            'title': 'test',
            'execution_date': date.today(),
            'association': association.id,
            'distance': '66',
            'project': project.id,
            'level': level.id,
            'category': category.id,
        }
        response = client.post(url, data, follow=True)
        assert response.status_code == 200
        assert count_text_in_content(response, "AssoTest")
        assert count_text_in_content(response, "ProjetTest")
        assert count_text_in_content(response, "NiveauTest")
        assert count_text_in_content(response, "CatégorieTest")

        messages = [m.message for m in get_messages(response.wsgi_request)]
        assert len(messages) == 1
        assert "a été modifié" in messages[0]

        benevalos = Benevalo.objects.all()
        assert len(benevalos) == 1
        assert Version.objects.get_for_object(benevalos[0]).count() == 2

        # 3. on vérifie la date dans le formulaire  (BUG #20)
        response = client.get(url, follow=True)
        execution_date = (
            bs(response.content, 'html.parser')
            .find(id='id_execution_date')
            .attrs['value']
        )
        assert execution_date == date.today().strftime('%Y-%m-%d')

        # 4. on delete
        url = (
            bs(response.content, 'html.parser')
            .find('a', string=RE_DELETE_BTN)
            .attrs['href']
        )
        data = {'confirm': True}
        response = client.post(url, data, follow=True)
        assert response.status_code == 200
        table_row = (
            bs(response.content, 'html.parser').find('tbody').find('tr')
        )
        assert len(table_row.contents) == 1
        assert len(table_row.contents[0].contents) == 1
        assert "Aucune donnée disponible." == table_row.contents[0].contents[0]
        messages = [m.message for m in get_messages(response.wsgi_request)]
        assert len(messages) == 1
        assert "a été supprimé" in messages[0]
