from datetime import timedelta

from django import forms
from django.core.exceptions import ValidationError
from django.forms.renderers import DjangoTemplates

import pytest

from ..fields import SplitDurationField, SplitDurationWidget


class TestSplitDurationField:
    @pytest.mark.parametrize(
        'value, hours, minutes',
        (
            ([2, 30], 2, 30),
            ([2, 90], 3, 30),
            # can be zero
            ([0, 0], 0, 0),
            ([2, 0], 2, 0),
            ([0, 30], 0, 30),
            # hours or minutes can be empty
            ([1, None], 1, 0),
            ([None, 30], 0, 30),
        ),
    )
    def test_clean(self, value, hours, minutes):
        f = SplitDurationField()
        assert f.clean(value) == timedelta(hours=hours, minutes=minutes)

    @pytest.mark.parametrize(
        'value, code',
        (
            (None, 'required'),
            ('', 'required'),
            (['', ''], 'required'),
            ([None, None], 'required'),
            ('hello', 'invalid'),
            (['hello', 'there'], ['invalid', 'invalid']),
            ([2, 'there'], ['invalid']),
            (['hello', 30], ['invalid']),
        ),
    )
    def test_validation_error(self, value, code):
        f = SplitDurationField()
        with pytest.raises(ValidationError) as excinfo:
            f.clean(value)
        if isinstance(code, list):
            assert [e.code for e in excinfo.value.error_list] == code
        else:
            assert excinfo.value.code == code

    @pytest.mark.parametrize('value', (None, '', [''], ['', '']))
    def test_optional_none(self, value):
        f = SplitDurationField(required=False)
        f.clean(value) is None

    def test_require_all_fields(self):
        f = SplitDurationField(require_all_fields=True)
        with pytest.raises(ValidationError) as excinfo:
            f.clean([1, None])
        assert excinfo.value.code == 'required'

    def test_custom_error_messages(self):
        f = SplitDurationField(error_messages={'invalid_hours': "Oops"})
        with pytest.raises(ValidationError) as excinfo:
            f.clean(['hello', 30])
        assert excinfo.value.error_list[0].message == "Oops"

    def test_min_value(self):
        f = SplitDurationField(min_value=None)
        f.clean([-1, 0])
        # min_value is set to zero by default
        f = SplitDurationField()
        with pytest.raises(ValidationError) as excinfo:
            f.clean([-1, 0])
        assert excinfo.value.error_list[0].code == 'min_value'

    def test_max_value(self):
        f = SplitDurationField(max_value=timedelta(hours=1))
        with pytest.raises(ValidationError) as excinfo:
            f.clean([0, 61])
        assert excinfo.value.error_list[0].code == 'max_value'

    def test_custom_widget(self):
        class TestWidget(SplitDurationWidget):
            pass

        f = SplitDurationField(widget=TestWidget)
        assert 'min' in f.widget.widgets[0].attrs

        f = SplitDurationField(widget=forms.TextInput)
        assert 'min' not in f.widget.attrs


class TestSplitDurationWidget:
    widget = SplitDurationWidget()

    def render(self, widget, name, value, attrs=None):
        return widget.render(
            name, value, attrs=attrs, renderer=DjangoTemplates()
        )

    def test_field_widget(self):
        f = SplitDurationField()
        assert len(f.fields) == 2
        assert isinstance(f.fields[0].widget, forms.NumberInput)
        assert isinstance(f.fields[1].widget, forms.NumberInput)
        assert isinstance(f.widget, SplitDurationWidget)
        widgets = f.widget.widgets
        assert len(widgets) == 2
        assert isinstance(widgets[0], forms.NumberInput)
        assert isinstance(widgets[1], forms.NumberInput)

    def test_field_default_attr(self):
        f = SplitDurationField()
        widgets = f.widget.widgets
        assert widgets[0].attrs['min'] == 0
        assert widgets[1].attrs['min'] == 0
        assert 'max' not in widgets[0].attrs
        assert 'max' not in widgets[1].attrs

    def test_field_min_attr(self):
        f = SplitDurationField(min_value=timedelta(minutes=70))
        widgets = f.widget.widgets
        assert widgets[0].attrs['min'] == 1
        assert widgets[1].attrs['min'] == 0

        # check with negative value
        f = SplitDurationField(min_value=timedelta(minutes=-70))
        widgets = f.widget.widgets
        assert widgets[0].attrs['min'] == -1
        assert widgets[1].attrs['min'] == -70

    def test_field_max_attr(self):
        f = SplitDurationField(max_value=timedelta(minutes=70))
        widgets = f.widget.widgets
        assert widgets[0].attrs['max'] == 1
        assert widgets[1].attrs['max'] == 70

        # check with negative value
        f = SplitDurationField(max_value=timedelta(minutes=-70))
        widgets = f.widget.widgets
        assert widgets[0].attrs['max'] == -1
        assert widgets[1].attrs['max'] == -70

    def test_render_empty(self):
        output = self.render(self.widget, 'duration', '')
        assert '<input type="number" name="duration_0">' in output
        assert '<input type="number" name="duration_1">' in output

    def test_render_timedelta(self):
        output = self.render(self.widget, 'duration', timedelta(hours=2))
        assert '<input type="number" name="duration_0" value="2">' in output
        assert '<input type="number" name="duration_1" value="0">' in output

    def test_render_constructor_attrs(self):
        widget = SplitDurationWidget(attrs={'class': 'foo'})
        output = self.render(widget, 'duration', None)
        assert '<input type="number" name="duration_0" class="foo">' in output
        assert '<input type="number" name="duration_1" class="foo">' in output

    @pytest.mark.parametrize(
        'widget_kwargs',
        (
            {
                'hours_attrs': {'class': 'foo'},
                'minutes_attrs': {'class': 'bar'},
            },
            {'hours_attrs': {'class': 'foo'}, 'attrs': {'class': 'bar'}},
            {'minutes_attrs': {'class': 'bar'}, 'attrs': {'class': 'foo'}},
        ),
    )
    def test_render_constructor_different_attrs(self, widget_kwargs):
        widget = SplitDurationWidget(**widget_kwargs)
        output = self.render(widget, 'duration', None)
        assert '<input type="number" name="duration_0" class="foo">' in output
        assert '<input type="number" name="duration_1" class="bar">' in output
